// Arrow.cpp
// Brian Seney, Alfredo Porras
// IMGD3000

#include "Arrow.h"
#include "Engine/WorldManager.h"
#include "Engine/GraphicsManager.h"
#include "Engine/LogManager.h" //

Arrow::Arrow(int x, int y, float x_vel, float y_vel){
	object_type = BULLET_TYPE;

	damage = 1;
	time_to_live = 30;

	velocity_x = x_vel;
	velocity_y = y_vel;

	display_char = '+';
	color = 0;

	EventManager &event_manager = EventManager::getInstance();
	event_manager.registerInterest(this, STEP_EVENT);
	event_manager.registerInterest(this, COLLISION_EVENT);

	WorldManager &world_manager = WorldManager::getInstance();
	world_manager.addObj(this);
	world_manager.moveObj(this, Position(x, y));
}

Arrow::~Arrow(){
	EventManager &event_manager = EventManager::getInstance();
	event_manager.removeInterest(this, STEP_EVENT);
	event_manager.removeInterest(this, COLLISION_EVENT);
}

int Arrow::eventHandler(Event *event_ptr){
	if(event_ptr->getType().compare(STEP_EVENT) == 0){
		velocityStep();
		time_to_live--;
		if(time_to_live == 0){
			WorldManager &world_manager = WorldManager::getInstance();
			world_manager.markForDelete(this);
		}
		return 1;
	} else if(event_ptr->getType().compare(COLLISION_EVENT) == 0){
		// if the other shot is an arrow, move anyways
		EventCollision *collision_ptr = static_cast<EventCollision*>(event_ptr);
		if(collision_ptr->getObj2()->getType().compare(BULLET_TYPE) == 0){
			setPos(collision_ptr->getPos()); 
			return 1;
		}
		WorldManager &world_manager = WorldManager::getInstance();
		world_manager.markForDelete(this);
		return 1;
	}
	return 0;
}
