// Ball.cpp
// Brian Seney, Alfredo Porras
// IMGD3000

#include "Ball.h"

#include "Engine/EventManager.h"
#include "Engine/EventStep.h"
#include "Engine/EventCollision.h"
#include "Engine/GraphicsManager.h"
#include "Engine/WorldManager.h"

#include "EventScore.h"
#include "Opponent.h"
#include "Goal.h"
#include "EventBallSteal.h"
#include "EventBallMovement.h"
#include "EnemyArrow.h"

#define BALL_COLOR 6

#define sign(a) a > 0

Ball::Ball(){
	is_solid = true;
	object_type = BALL_TYPE;

	WorldManager &world_manager = WorldManager::getInstance();
	world_manager.addObj(this);

	EventManager &event_manager = EventManager::getInstance();
	event_manager.registerInterest(this, STEP_EVENT);
	event_manager.registerInterest(this, COLLISION_EVENT);
}

Ball::~Ball(){
	EventManager &event_manager = EventManager::getInstance();
	event_manager.removeInterest(this, STEP_EVENT);
	event_manager.removeInterest(this, COLLISION_EVENT);

	// Inform any player holding this ball that it must be released.
	event_manager.sendEvent( new EventBallSteal(this) );
}

int Ball::eventHandler(Event *event_ptr){
	if(event_ptr->getType().compare(STEP_EVENT) == 0){
		if(sign(velocity_x + accel_x) != sign(velocity_x) ||
		   sign(velocity_y + accel_y) != sign(velocity_y)){
			// if true, the object is switching from a positive to negative velocity.
			// in other words, the object should slow and be stopping.
			velocity_x = 0;
			velocity_y = 0;
			accel_x = 0;
			accel_y = 0;
		}
		velocityStep();
		if(velocity_x != 0 || velocity_y != 0){
			// if moving, let objects know where the ball is
			EventManager &event_manager = EventManager::getInstance();
			event_manager.sendEvent( new EventBallMovement(pos) );
		}
		return 1;
	} else if(event_ptr->getType().compare(COLLISION_EVENT) == 0){
		EventCollision *collision_ptr = static_cast<EventCollision*>(event_ptr);
		// If colliding with the goal, it's a goal!
		GameObject* obj_ptr = collision_ptr->getObj2();
		if(obj_ptr->getType().compare( GOAL_TYPE ) == 0 ){
			WorldManager &world_manager = WorldManager::getInstance();
			world_manager.markForDelete(this);

			EventManager &event_manager = EventManager::getInstance();
			event_manager.sendEvent( new EventScore(2) );	
			// Inform any player holding this ball that it must be released.
			event_manager.sendEvent( new EventBallSteal(this) );
		}
		return 1;
	}
	return 0;
}

void Ball::draw(){
	GraphicsManager &graph_manager = GraphicsManager::getInstance();
	graph_manager.drawCh(pos, 'O', BALL_COLOR );
}

int Ball::moveAbsolute(int x, int y){
	WorldManager &world_manager = WorldManager::getInstance();
	int return_val = world_manager.moveObj(this, Position(x, y));
	if(return_val == 0){
		EventManager &event_manager = EventManager::getInstance();
		event_manager.sendEvent( new EventBallMovement(pos) );
	}
	return return_val;
}

int Ball::move(int x, int y){
	WorldManager &world_manager = WorldManager::getInstance();
	int return_val = world_manager.moveObj(this, Position(pos.get_x() + x, pos.get_y() + y));
	if(return_val == 0){
		EventManager &event_manager = EventManager::getInstance();
		event_manager.sendEvent( new EventBallMovement(pos) );
	}
	return return_val;
}
