// Bomb.cpp
// Brian Seney, Alfredo Porras
// IMGD3000

#include "Bomb.h"
#include "Arrow.h"
#include "Engine/WorldManager.h"
#include "Engine/GraphicsManager.h"

Bomb::Bomb(int x, int y, float x_vel, float y_vel){
	object_type = BULLET_TYPE;

	damage = 1;
	time_to_live = 100;

	velocity_x = x_vel;
	velocity_y = y_vel;

	display_char = 'Y';
	color = 0;

	EventManager &event_manager = EventManager::getInstance();
	event_manager.registerInterest(this, STEP_EVENT);
	event_manager.registerInterest(this, COLLISION_EVENT);

	WorldManager &world_manager = WorldManager::getInstance();
	world_manager.addObj(this);
	world_manager.moveObj(this, Position(x, y));
}

Bomb::~Bomb(){
	EventManager &event_manager = EventManager::getInstance();
	event_manager.removeInterest(this, STEP_EVENT);
	event_manager.removeInterest(this, COLLISION_EVENT);

	// Create an arrow in the eight cardinal directions
	new Arrow(pos.get_x() - 1, pos.get_y() - 1, -1, -1);
	new Arrow(pos.get_x(), pos.get_y() - 1, 0, -1);
	new Arrow(pos.get_x() + 1, pos.get_y() - 1, 1, -1);

	new Arrow(pos.get_x() - 1, pos.get_y(), -1, 0);
	new Arrow(pos.get_x() + 1, pos.get_y(), 1, 0);

	new Arrow(pos.get_x() - 1, pos.get_y() + 1, -1, 1);
	new Arrow(pos.get_x(), pos.get_y(), 0, 1);
	new Arrow(pos.get_x() + 1, pos.get_y() + 1, 1, 1);
}

int Bomb::eventHandler(Event *event_ptr){
	if(event_ptr->getType().compare(STEP_EVENT) == 0){
		time_to_live--;
		if(time_to_live == 0){
			WorldManager &world_manager = WorldManager::getInstance();
			world_manager.markForDelete(this);
		}
		velocityStep();
		return 1;
	} else if(event_ptr->getType().compare(COLLISION_EVENT) == 0){
		WorldManager &world_manager = WorldManager::getInstance();
		world_manager.markForDelete(this);
		return 1;
	}
	return 0;
}
