// EnemyArrow.cpp
// Brian Seney, Alfredo Porras
// IMGD3000

#include "EnemyArrow.h"
#include "Arrow.h"
#include "Engine/GraphicsManager.h"

EnemyArrow::EnemyArrow(int x, int y, float x_vel, float y_vel){
	object_type = ENEMY_ARROW_TYPE;

	damage = 1;

	time_to_live = 50;

	velocity_x = x_vel;
	velocity_y = y_vel;

	velocity_step_x = 0;
	velocity_step_y = 0;

	display_char = '+';

	EventManager &event_manager = EventManager::getInstance();
	event_manager.registerInterest(this, STEP_EVENT);
	event_manager.registerInterest(this, COLLISION_EVENT);

	WorldManager &world_manager = WorldManager::getInstance();
	world_manager.addObj(this);
	world_manager.moveObj(this, Position(x, y));
}

EnemyArrow::~EnemyArrow(){
	EventManager &event_manager = EventManager::getInstance();
	event_manager.removeInterest(this, STEP_EVENT);
	event_manager.removeInterest(this, COLLISION_EVENT);
}

int EnemyArrow::eventHandler(Event *event_ptr){
	if(event_ptr->getType().compare(STEP_EVENT) == 0){
		velocityStep();
		time_to_live--;
		if(time_to_live == 0){
			WorldManager &world_manager = WorldManager::getInstance();
			world_manager.markForDelete(this);
		}
		return 1;
	} else if(event_ptr->getType().compare(COLLISION_EVENT) == 0){
		// if the other shot is an enemy arrow, or a regular arrow, move anyways
		EventCollision *collision_ptr = static_cast<EventCollision*>(event_ptr);
		if(collision_ptr->getObj2()->getType().compare(ENEMY_ARROW_TYPE) == 0 ||
		   collision_ptr->getObj2()->getType().compare(BULLET_TYPE) == 0){
			setPos(collision_ptr->getPos()); 
			return 1;
		}
		WorldManager &world_manager = WorldManager::getInstance();
		world_manager.markForDelete(this);
		return 1;
	}
	return 0;
}

void EnemyArrow::draw(){
	GraphicsManager &graph_manager = GraphicsManager::getInstance();
	graph_manager.drawCh(pos, display_char, (time_to_live % 2) ? 2 : 3);
}
