// EventManager.cpp
// Brian Seney
// IMGD3000

#include "EventManager.h"
#include "LogManager.h"

int EventManager::startUp(){
	is_started = true;
	LogManager &log_manager = LogManager::getInstance();
	log_manager.writeLog(1, "Event Manager: Started!");
}

int EventManager::shutDown(){
	is_started = false;
}

void EventManager::sendEvent(Event* e){
	LogManager &log_manager = LogManager::getInstance();
	log_manager.writeLog(3, " Event sent: %s", e->getType().c_str());
	// Determine the type of event received
	string temp = e->getType();
	// Determine if any objects are registered for the given event
	int i = 0;
	for(; i < num_events; i++){
		if(temp.compare(event_name[i]) == 0){
			break; // i is at the correct index
		}
	}

	if(i == num_events){
		return; // the string was not found
	}

	// Send the event to all objects in the object list
	list<GameObject*>::iterator iter = (obj_list[i]).begin();
	while(iter != (obj_list[i]).end()){
		(*iter)->eventHandler(e);
		iter++;
	}
}

void EventManager::registerInterest(GameObject* obj_ptr, string event_type){
	LogManager &log_manager = LogManager::getInstance();
	log_manager.writeLog(3, "Interest registered: object %i, %s", obj_ptr->getId(), event_type.c_str());

	// Determine if the event is already present
	int i = 0;
	for(; i < num_events; i++){
		if(event_type.compare(event_name[i]) == 0){
			// The event is present, add the object to the corresponding list
			obj_list[i].add(obj_ptr);
			return;
		}
	}
	// If reached here, the event is not present. Add a new event
	event_name[num_events] = event_type;
	obj_list[num_events].add(obj_ptr);
	num_events++;
}

void EventManager::removeInterest(GameObject* obj_ptr, string event_type){
	LogManager &log_manager = LogManager::getInstance();
	log_manager.writeLog(2, "Interest removed: object %i, %s", obj_ptr->getId(), event_type.c_str());
	// Determine which event to remove
	int i = 0;
	for(; i < num_events; i++){
		if(event_type.compare(event_name[i]) == 0){
			// The event is present, remove the object from the corresponding list
			obj_list[i].remove(obj_ptr);
			return;
		}
	}
	// If reached here, the event is not present. Nothing needs to be done in this case
}
