// GameObject.cpp
// Brian Seney
// IMGD3000

#include "GameObject.h"
#include "GameManager.h"
#include "LogManager.h"
#include "WorldManager.h"
#include <math.h>

GameObject::GameObject(){
	numObjs++;
	id = numObjs;
	altitude = 0;
	pos = Position(0, 0);
	obj_box = Box(pos, 1, 1);
	is_solid = true;

	velocity_step_x = 0;
	velocity_step_y = 0;
	velocity_x = 0;
	velocity_y = 0;
	accel_x = 0;
	accel_y = 0;
}

GameObject::~GameObject(){
}

void GameObject::setPos(Position new_pos){
	pos = new_pos;
	obj_box.setPos(new_pos);
}

void GameObject::setPos(int new_x, int new_y){
	Position temp_pos = Position(new_x, new_y);
	pos = Position(temp_pos);
	obj_box.setPos(temp_pos);
}

void GameObject::setAltitude(int alt){
	if( (-1 * MAX_ALTITUDE) <= alt && alt <= MAX_ALTITUDE)
		altitude = alt;
	else
		altitude = 0;
}

void GameObject::velocityStep(){
	// Increment the velocity by the acceleration
	velocity_x += accel_x;
	velocity_y += accel_y;
	// Increment the position by the velocity
	velocity_step_x += velocity_x;
	velocity_step_y += velocity_y;

	int temp;
	int temp_x = pos.get_x();
	int temp_y = pos.get_y();
	bool changed = false;
	// If the step is over 1 (or under -1), update the position and step counters
	if(velocity_step_x >= 1 || velocity_step_x <= -1){
		if(velocity_step_x >= 1)
			temp = (int)floor(velocity_step_x);
		else
			temp = (int)ceil(velocity_step_x);
		velocity_step_x -= temp;
		temp_x =  pos.get_x() + temp;
		changed = true;
	}
	if(velocity_step_y >= 1 || velocity_step_y <= -1){
		if(velocity_step_y >= 1)
			temp = (int)floor(velocity_step_y);
		else
			temp = (int)ceil(velocity_step_y);
		velocity_step_y -= temp;
		temp_y =  pos.get_y() + temp;
		changed = true;
	}
	// If the position changed, update the position
	if(changed){
		WorldManager &world_manager = WorldManager::getInstance();
		world_manager.moveObj(this, Position(temp_x, temp_y));
	}
}

int GameObject::eventHandler(Event *event_ptr){
	// Ignore the event
	return 0;
}

void GameObject::draw(){}
