// GameObject.h
// Brian Seney
// IMGD3000

#ifndef __GAMEOBJECT_H__
#define __GAMEOBJECT_H__

#include "Position.h"
#include "Box.h"
#include "Event.h"
#include "Sprite.h"
#include <string>

#define MAX_ALTITUDE 2 // range from -2 .. 2

using namespace std;

class GameObject{
	private:
	// Total number of objects created 
	static int numObjs;

	protected:
	int id;
	string object_type;
	//
	Position pos;
	Position old_pos;
	float velocity_x;
	float velocity_y;
	float velocity_step_x;
	float velocity_step_y;
	float accel_x;
	float accel_y;

	Box obj_box;
	bool is_solid;
	int altitude;

	Sprite *sprite_ptr;

	int sprite_index;
	int sprite_slowdown;
	int sprite_countdown;

	public:
	GameObject();
	virtual ~GameObject();
	
	// Getters, setters for protected fields
	void setId(int new_id){id = new_id;}
	int getId(){return id;}
	void setType(string new_type){object_type = new_type;}
	string getType(){return object_type;}
	void setSolid(bool solid){is_solid = solid;}
	bool isSolid(){return is_solid;}
	void setPos(Position new_pos); // Position, velocity, acceleration
	void setPos(int new_x, int new_y);
	void setOldPos(Position o_pos){old_pos = o_pos;}
	Position getPos(){return pos;}
	Position getOldPos(){return old_pos;}
	void setVelocity(float new_x, float new_y){
		velocity_x = new_x;
		velocity_y = new_y;
	}
	void setVelocityX(float new_x){ velocity_x = new_x; }
	void setVelocityY(float new_y){ velocity_y = new_y; }
	void setAccel(float new_x, float new_y){
		accel_x = new_x;
		accel_y = new_y;
	}
	void setAccelX(float new_x){ accel_x = new_x; }
	void setAccelY(float new_y){ accel_y = new_y; }
	float getVelocityX(){ return velocity_x; }
	float getVelocityY(){ return velocity_y; }
	float getAccelX(){ return accel_x; }
	float getAccelY(){ return accel_y; }
	void setBox(Box new_box){obj_box = new_box;}
	Box getBox(){return obj_box;}
	void setAltitude(int alt);
	int getAltitude(){return altitude;}
	void setSprite(Sprite *new_sprite_ptr){sprite_ptr = new_sprite_ptr;}
	Sprite *getSprite(){return sprite_ptr;}
	void setSpriteIndex(int new_index){sprite_index = new_index;}
	int getSpriteIndex(){return sprite_index;}
	void setSpriteSlowdown(int new_slowdown){sprite_slowdown = new_slowdown;}
	int getSpriteSlowdown(){return sprite_slowdown;}
	void setSpriteSlowdownCount(int new_count){sprite_countdown = new_count;}
	int getSpriteSlowdownCount(){return sprite_countdown;}

	// Functions
	// Perform a velocity step; update the position if needed
	void velocityStep();
	// Handle the event (return 0 if ignored, else 1)
	virtual int eventHandler(Event *event_ptr);
	// Draw the object
	virtual void draw();	
};

int GameObject::numObjs = 0;

#endif // __GAMEOBJECT_H__
