// GraphicsManager.h
// Brian Seney
// IMGD3000

#ifndef __GRAPHICSMANAGER_H__
#define __GRAPHICSMANAGER_H__

#include <curses.h>

#include "Manager.h"
#include "Position.h"
#include "Frame.h"

class GraphicsManager : public Manager{
	private:
	// Two buffers for drawing
	WINDOW *win1;
	WINDOW *win2;
	WINDOW *current_window;
	// Maximum width and height of the window
	int window_x;
	int window_y;
	// A camera abstraction to move the drawing window
	Position camera;
	// Prohibit copy, assignment
	GraphicsManager(GraphicsManager const&){};
	void operator=(GraphicsManager const&){};

	// Private constructor, destructor
	GraphicsManager(){}
	~GraphicsManager(){}

	public:
	// Start, stop the graphics manager
	int startUp();
	int shutDown();

	// Return, set camera
	Position getCamera(){ return camera; }

	void setCamera(Position pos){ camera = pos; } // moves the camera to the given position
	void moveCamera(Position pos){ // adds the camera position to the given position (slides it over)
		camera.set_x( camera.get_x() + pos.get_x() );
		camera.set_y( camera.get_y() + pos.get_y() );
	}

	// Return the max x and y window size
	int getMaxX(){ return window_x; }
	int getMaxY(){ return window_y; }

	// Convert world position to screen position
	Position worldToScreen(Position pos);

	// Flush the buffer to the screen.
	void swapBuffers();

	// Draw a character at the given position (with optional color)
	int drawCh(Position pos, char ch);
	int drawCh(Position pos, char ch, int color);

	// Draw a string at the given position
	int drawStr(Position pos, char* ch);

	// Draw a frame at the given location
	int drawFrame(Position pos, Frame *frame_ptr);

	// Return horizontal, vertical maximums
	int getHorizontal(){ return window_x; }
	int getVertical(){ return window_y; }

	// Return the GraphicsManager instance
	static GraphicsManager& getInstance(){
		static GraphicsManager manager;
		return manager;
	}
};

#endif // __GRAPHICSMANAGER_H__
