// InputManager.cpp
// Brian Seney
// IMGD3000

#include "InputManager.h"
#include "GameManager.h"
#include "EventManager.h"
#include "GraphicsManager.h"
#include "LogManager.h"
#include "Event.h"
#include "EventKeyboard.h"
#include "EventMouse.h"

#include <curses.h>

int InputManager::startUp(){
	is_started = true;
	LogManager &log_manager = LogManager::getInstance();
	log_manager.writeLog(1, "Input Manager: Started!");

	//enable keypad
	keypad(stdscr, TRUE);
	//disable line buffering
	cbreak();
	// turnoff newline
	nonl();
	//disable character echo
	noecho();
	// turn off cursor
	curs_set(0);
	// set no delay
	nodelay(stdscr, TRUE); // what's window?
	// enable mouse events
	mmask_t mask = BUTTON1_CLICKED | BUTTON2_CLICKED | BUTTON1_DOUBLE_CLICKED | BUTTON2_DOUBLE_CLICKED;
	mousemask(mask, NULL);

	return 0;
}

int InputManager::shutDown(){
	is_started = false;
	curs_set(1);
	return 0;
}

void InputManager::getInput(){
	// Make sure the graphics manager is started
	// (and therefore curses is initialized)
	GraphicsManager &graphics_manager = GraphicsManager::getInstance();
	if(graphics_manager.isStarted() ){
		// Send the keyboard input to all interested object
		int c = getch();
		// While there is still input
		while(c != ERR){
			LogManager &log_manager = LogManager::getInstance(); //
			// If c is mouse input
			if(c == KEY_MOUSE){
				MEVENT mouse_event;
				if(getmouse(&mouse_event) == OK){
			   		log_manager.writeLog(3, " Mouse event found!");
			   		EventMouse* e = new EventMouse();
			   		e->setMousePosition(Position(mouse_event.x, mouse_event.y));
			    		if(mouse_event.bstate & BUTTON1_CLICKED){
			      			log_manager.writeLog(3, " - left click!");
			      			e->setMouseAction( LEFT_CLICK );
			    		} else if(mouse_event.bstate & BUTTON1_DOUBLE_CLICKED){
			      			log_manager.writeLog(3, " - left double click!");
			      			e->setMouseAction( RIGHT_CLICK );
			    		} else if(mouse_event.bstate & BUTTON3_CLICKED){
			     			log_manager.writeLog(3, " - right click!");
			      			e->setMouseAction( RIGHT_CLICK );
                            		} else if(mouse_event.bstate & BUTTON3_DOUBLE_CLICKED){
                              			log_manager.writeLog(3, " - right double click!");
			      			e->setMouseAction( RIGHT_DOUBLECLICK );
                            		}

					EventManager &event_manager = EventManager::getInstance();
					event_manager.sendEvent( e );
			  	}
			// If c is keyboard input
			}else{
				EventKeyboard* e = new EventKeyboard();
				e->setKey(c);

				EventManager &event_manager = EventManager::getInstance();
				event_manager.sendEvent( e );
			}
			c = getch();
		}
	}
}
