// LogManager.cpp
// Brian Seney, Alfredo Porras
// IMGD 3000

#include "LogManager.h"

// Set up the debug level of the log manager
// Open the file for writing
int LogManager::startUp(int debug, bool append){
	debug_level = debug;
	// Open the file in the correct mode
	if(append)
		logfile.open("dragonfly.txt", ios_base::app);
	else
		logfile.open("dragonfly.txt", ios_base::out);
	// Verify the file is open
	if(!logfile.is_open() ){
		printf("ERROR: The log file failed to open.");
		is_started = false;
		return 1;
	}
	// Print an initialization message
	logfile << "**** START ****\n";
	logfile << " Dragonfly Version 1.0 - Created by Brian Seney, Alfredo Porras\n\n";
	writeLog(0, "Log Manager: Started!");

	is_started = true;

	return 0;
}

// On shutdown close the file
int LogManager::shutDown(){
	is_started = false;
	// Print a closing message
	writeLog(0, "Log File: Closed\n***** END *****\n");
	logfile.close();


	return 0;
}

// Write to the file
void LogManager::writeLog(int debug, char* str, ...){
	if(debug <= debug_level){
		logfile << getTime() << ": ";
		// Obtain a string representing all the arguments
		va_list args;
		va_start(args, str);
		char temp_str[256];
		// Format the obtained string
		vsprintf(temp_str, str, args);
		logfile << temp_str << endl;
		
		va_end(args);
	}
}

// Obtain a string representing the current time (see time.h)
string LogManager::getTime(){
	time_t epoch_time = time(NULL);
	tm *current_time = localtime(&epoch_time);
	string str = asctime(current_time);
	// Erase the trailing newline from the obtained string
	str.erase( str.find("\n", 1) );

	return str;
}
