// Alfredo Porras
// Brian Seney
// IMGD 3000

// Path Search

#include "PathSearch.h"
#include "PathSearchNode.cpp"
#include "LogManager.h"

PathSearch::PathSearch() {

}

PathSearch::~PathSearch() {

}

void PathSearch::findPath(Position start, Position end, stack<Position> *p_path_stack) {
//	LogManager &log_manager = LogManager::getInstance();
//	log_manager.writeLog(1,"PathSearch::findPath(): Hi goal = %d %d.", end.get_x(), end.get_y());
	goal = end;
	
	PathSearchNode *p_start_node = new PathSearchNode(NULL, start, 0, goal);
	fringe.push(*p_start_node);
	visited.push_back(p_start_node);
	
	PathSearchNode current_node = fringe.top();
	Position current_pos = current_node.getPos(); // instead of start
	while(!isGoal(current_pos)) {
//		log_manager.writeLog(1,"PathSearch::findPath(): looping current %d %d f:%fL. q = %d", current_pos.get_x(), current_pos.get_y(),current_node.getEstimate(), fringe.size());

		if (!fringe.empty()) {
			fringe.pop(); //  calls destructor, hope this doesn't break...
		}
		else {
			cleanVisited();
			return; // need to handle this. It means nothing was pushed onto p_path_stack;
		}
		
		expand(current_node);
		current_node = (fringe.top());
		current_pos = (current_node.getPos());
	}
	// hopefully: current_node == fringe.top() == goal node
	
//	log_manager.writeLog(1,"PathSearch::findPath(): Hi3.");
	PathSearchNode path_node = current_node;
//	log_manager.writeLog(1,"						%d %d", path_node.getPos().get_x(),path_node.getPos().get_y());
	PathSearchNode *p_path_node = &path_node;
	while(p_path_node != NULL) {
		p_path_stack->push(p_path_node->getPos());
//		log_manager.writeLog(1," pushing pos %d %d with parent d d and d d", p_path_node->getPos().get_x(), p_path_node->getPos().get_y()/*, p_path_node->getParent()->getPos().get_x(),p_path_node->getParent()->getPos().get_y(), p_path_node->getParent()->getParent()->getPos().get_x(),p_path_node->getParent()->getParent()->getPos().get_y()*/);
		p_path_node = p_path_node->getParent();
	}
	cleanVisited();
//	log_manager.writeLog(1,"PathSearch::findPath(): Hi4.");
}

void PathSearch::expand(PathSearchNode node) {
	WorldManager &world_manager = WorldManager::getInstance();
	LogManager &log_manager = LogManager::getInstance();
//	log_manager.writeLog(1, "PathSearch::expand(): expanding %d %d.", node.getPos().get_x(), node.getPos().get_y());
	vector<PathSearchNode *>::iterator vit;
	
	if (world_manager.isPassable(node.getPos().get_x() + 1,node.getPos().get_y()) ||
		isGoal(Position(node.getPos().get_x() + 1,node.getPos().get_y()))){
//		log_manager.writeLog(1, "PathSearch::expand(): right is good.");
//		PathSearchNode *p_new_node = new PathSearchNode(&node, Position(node.getPos().get_x() + 1,node.getPos().get_y()), node.getG() + 1, goal);
		Position *p_new_node_pos = new Position(node.getPos().get_x() + 1,node.getPos().get_y());
		bool not_visited = true;
		for( vit = visited.begin(); vit < visited.end(); vit++) {
			if (((*vit)->getPos().get_x() == p_new_node_pos->get_x()) &&
				((*vit)->getPos().get_y() == p_new_node_pos->get_y())) {
				not_visited = false;
				break;
			}
		}
		if(not_visited) {
//			log_manager.writeLog(1, "PathSearch::expand(): pushing.");
			PathSearchNode *p_new_node = new PathSearchNode(visitedNode(node), *p_new_node_pos, node.getG() + 1, goal);
//			log_manager.writeLog(1, "PathSearch::expand(): parent is %d %d.",p_new_node->getParent()->getPos().get_x(),p_new_node->getParent()->getPos().get_y());
			fringe.push(*p_new_node);
			visited.push_back(p_new_node);
		}/* else {log_manager.writeLog(1, "PathSearch::expand(): not pushing.");}*/
		delete p_new_node_pos;
	}/* else {log_manager.writeLog(1, "PathSearch::expand(): right not good.");}*/
	if (world_manager.isPassable(node.getPos().get_x(),node.getPos().get_y() + 1) ||
		isGoal(Position(node.getPos().get_x(),node.getPos().get_y() + 1))){
//		log_manager.writeLog(1, "PathSearch::expand(): down is good.");
//		PathSearchNode *p_new_node = new PathSearchNode(&node, Position(node.getPos().get_x(),node.getPos().get_y() + 1), node.getG() + 1, goal);
		Position *p_new_node_pos = new Position(node.getPos().get_x(),node.getPos().get_y() + 1);
		bool not_visited = true;
		for( vit = visited.begin(); vit < visited.end(); vit++) {
			if (((*vit)->getPos().get_x() == p_new_node_pos->get_x()) &&
				((*vit)->getPos().get_y() == p_new_node_pos->get_y())) {
				not_visited = false;
				break;
			}
		}
		if(not_visited) {
//			log_manager.writeLog(1, "PathSearch::expand(): pushing.");
			PathSearchNode *p_new_node = new PathSearchNode(visitedNode(node), *p_new_node_pos, node.getG() + 1, goal);
//			log_manager.writeLog(1, "PathSearch::expand(): parent is %d %d.",p_new_node->getParent()->getPos().get_x(),p_new_node->getParent()->getPos().get_y());
			fringe.push(*p_new_node);
			visited.push_back(p_new_node);
		}/* else {log_manager.writeLog(1, "PathSearch::expand(): not pushing.");}*/
		delete p_new_node_pos;
	}/*else {log_manager.writeLog(1, "PathSearch::expand(): down not good.");}*/
	if (world_manager.isPassable(node.getPos().get_x() - 1,node.getPos().get_y()) ||
		isGoal(Position(node.getPos().get_x() - 1,node.getPos().get_y()))){
//		log_manager.writeLog(1, "PathSearch::expand(): left is good.");
//		PathSearchNode *p_new_node = new PathSearchNode(&node, Position(node.getPos().get_x() - 1,node.getPos().get_y()), node.getG() + 1, goal);
		Position *p_new_node_pos = new Position(node.getPos().get_x() - 1,node.getPos().get_y());
		bool not_visited = true;
		for( vit = visited.begin(); vit < visited.end(); vit++) {
			if (((*vit)->getPos().get_x() == p_new_node_pos->get_x()) &&
				((*vit)->getPos().get_y() == p_new_node_pos->get_y())) {
				not_visited = false;
				break;
			}
		}
		if(not_visited) {
//			log_manager.writeLog(1, "PathSearch::expand(): pushing.");
			PathSearchNode *p_new_node = new PathSearchNode(visitedNode(node), *p_new_node_pos, node.getG() + 1, goal);
//			log_manager.writeLog(1, "PathSearch::expand(): parent is %d %d.",p_new_node->getParent()->getPos().get_x(),p_new_node->getParent()->getPos().get_y());
			fringe.push(*p_new_node);
			visited.push_back(p_new_node);
		}/* else {log_manager.writeLog(1, "PathSearch::expand(): not pushing.");}*/
		delete p_new_node_pos;
	}/*else {log_manager.writeLog(1, "PathSearch::expand(): left not good.");}*/
	if (world_manager.isPassable(node.getPos().get_x(),node.getPos().get_y() - 1) ||
		isGoal(Position(node.getPos().get_x(),node.getPos().get_y() - 1))){
//		log_manager.writeLog(1, "PathSearch::expand(): up is good.");
//		PathSearchNode *p_new_node = new PathSearchNode(&node, Position(node.getPos().get_x(),node.getPos().get_y() - 1), node.getG() + 1, goal);
		Position *p_new_node_pos = new Position(node.getPos().get_x(),node.getPos().get_y() - 1);
		bool not_visited = true;
		for( vit = visited.begin(); vit < visited.end(); vit++) {
			if (((*vit)->getPos().get_x() == p_new_node_pos->get_x()) &&
				((*vit)->getPos().get_y() == p_new_node_pos->get_y())) {
				not_visited = false;
				break;
			}
		}
		if(not_visited) {
//			log_manager.writeLog(1, "PathSearch::expand(): pushing.");
			PathSearchNode *p_new_node = new PathSearchNode(visitedNode(node), *p_new_node_pos, node.getG() + 1, goal);
//			log_manager.writeLog(1, "PathSearch::expand(): parent is %d %d.",p_new_node->getParent()->getPos().get_x(),p_new_node->getParent()->getPos().get_y());
			fringe.push(*p_new_node);
			visited.push_back(p_new_node);
		} /*else {log_manager.writeLog(1, "PathSearch::expand(): not pushing.");}*/
		delete p_new_node_pos;
	}/*else {log_manager.writeLog(1, "PathSearch::expand(): up not good.");}*/
}

bool PathSearch::isGoal(Position current) {
	if ((current.get_x() == goal.get_x()) &&
		(current.get_y() == goal.get_y())) {
		return true;
	}
	return false;
}

PathSearchNode *PathSearch::visitedNode(PathSearchNode n) {
	vector<PathSearchNode *>::iterator vit;
	for( vit = visited.begin(); vit < visited.end(); vit++) {
		if (((*vit)->getPos().get_x() == n.getPos().get_x()) &&
			((*vit)->getPos().get_y() == n.getPos().get_y())) {
			return (*vit);
		}
	}
	return NULL;
}

void PathSearch::cleanVisited() {
	vector<PathSearchNode *>::iterator vit;
	for( vit = visited.begin(); vit < visited.end(); vit++) {
		delete (*vit);
	}
}
