// ResourceManager.cpp
// Brian Seney
// IMGD3000

#include "ResourceManager.h"
#include "LogManager.h"

#include <cstring>
#include <string>
#include <fstream>
#include <vector>

using namespace std;

int ResourceManager::startUp(){
	is_started = true;
	LogManager &log_manager = LogManager::getInstance();
	log_manager.writeLog(1, "Resource Manager: Started!");
	return 0;
}

int ResourceManager::shutDown(){
	is_started = false;
	return 0;
}

Sprite* ResourceManager::loadSprite(string filename){
	string line;
	ifstream spr_file(filename.c_str());
	if(spr_file.is_open()){
		// Create the new sprite object
		Sprite* spr = new Sprite();
		spr->setLabel(filename);
		// Obtain the header fields in the files
		int numFrames = line_to_int(&spr_file, "frames");
		int width = line_to_int(&spr_file, "width");
		int height = line_to_int(&spr_file, "height");
		spr->setWidth(width);
		spr->setHeight(height);
		// Obtain the strings to print for each sprite
		for(int i = 0; i < numFrames; i++){
			spr->addFrame( frame_read(&spr_file, width, height) );
		}
		// Obtain the colors for each sprite
		for(int i = 0; i < numFrames; i++){
			Frame* spr_frame = spr->getFrame(i);
			spr_frame->setColors( *(frame_color_read(&spr_file, width, height)));
		}
		return spr;
	}
}

// Example: if "frames 3" is the next line in 'file', and expect is "frames", this returns 3
int ResourceManager::line_to_int(ifstream* file, string expect){
	// if the expected string can be found
	if(file->good()){
		string str;
		getline(*file, str);
		if(str.find(expect) != -1){
			char temp[1000];
			strcpy(temp, str.c_str());
			strtok(temp, " \n"); // Remove the expected string
			char* result = strtok(NULL, " \n"); // Extract the integer present
			return atoi(result);
		} else {
			return -1;
		}
	} else {
		return -1;
	}
}

// Create a new frame, and read in the characters the frame will have
Frame *ResourceManager::frame_read(ifstream* file, int width, int height){
	// if the expected string can be found
	if(file->good()){
		char frame_str[2000] = {0};
		string temp_str;
		// determine if lines are ready to be read
		while(1){
			getline(*file, temp_str);
			// find either 'start' or 'end' to determine the next string to read
			if(temp_str.find("start") != -1){
				break;
			// if the token could not be found before the file ended, return
			} else if(!file->good()){
				return NULL;
			}
		}
		// read the tokens for the frame's string
		for(int i = 0; i < height; i++){
			// make sure the file is still good
			if(!file->good()){
				return NULL;
			}
			getline(*file, temp_str);
			char temp_char[2000];
			strcpy(temp_char, temp_str.c_str());
			strcat(frame_str, temp_char);
		}
		return new Frame(width, height, frame_str);
	}
	
	else return NULL;
}

// Take the provided frame, and read in the colors the frame will have
vector<int> *ResourceManager::frame_color_read(ifstream* file, int width, int height){
	vector<int> *color_vec = new vector<int>;
	if(file->good()){
		string temp_str;
		// determine if lines are ready to be read
		while(1){
			getline(*file, temp_str);
			// find either 'start' or 'end' to determine the next string to read
			if(temp_str.find("start") != -1){
				break;
			// if the token could not be found before the file ended, return
			} else if(!file->good()){
				return NULL;
			}
		}
		// read the tokens for the frame's string
		for(int i = 0; i < height; i++){
			// make sure the file is still good
			if(!file->good()){
				return NULL;
			}
			getline(*file, temp_str);
			char temp_char[2000];
			strcpy(temp_char, temp_str.c_str());
			// tokenize the string and add the integers to the vector
			char* temp = strtok(temp_char, " \n");
			while(1){
				if(temp){
					color_vec->push_back(atoi(temp));
				} else {
					break;
				}
				temp = strtok(NULL, " \n");
			}
		}
		return color_vec;
	} else {
		return NULL;
	}
}

