// WorldManager.h
// Brian Seney
// IMGD3000

#ifndef __WORLDMANAGER_H__
#define __WORLDMANAGER_H__

#include "Manager.h"
#include "LogManager.h"
#include "GameObjectList.h"
#include "Position.h"
#include "EventStep.h"
#include "Box.h"

#define MAP_WIDTH 80
#define MAP_HEIGHT 24

class WorldManager : public Manager{
	private:
	GameObjectList game_objs;
	GameObjectList rm_objs;

	GameObjectList permanent_objs;

	Box world_boundary;

	bool clear; // whether or not to destroy the list of game_objs next tick

	// Prohibit copy, assignment
	WorldManager(WorldManager const&){};
	void operator=(WorldManager const&){};

	// Private constructor, destructor
	WorldManager(){}
	~WorldManager(){}

	bool path_map[MAP_WIDTH][MAP_HEIGHT]; // true is passable, false is not

	public:
	// Start, stop the world manager
	int startUp();
	int shutDown();

	// Mark the object for deletion (these return 0 on success, 1  on failure)
	int markForDelete(GameObject* obj_ptr);
	void setClear(){ clear = true; } // marks every object for deletion

	// Adds, removes an object from the list of permanent objects
	int setPermanent(GameObject *obj_ptr);
	int unsetPermanent(GameObject *obj_ptr);
	// Add an object to the world
	int addObj(GameObject* obj_ptr);
	// Remove an object from the world
	int removeObj(GameObject* obj_ptr);
	// Move the given object to the new position
	int moveObj(GameObject* obj_ptr, Position new_pos);
	// Determine if there will be a collision at the given position
	int testCollision(Position obj_pos);

//->
	bool isPassable(int x, int y);
	void setPassable(int x, int y, bool pass);
	void resetPathMap();
//<-

	// Return a copy of the GameObjectList
	GameObjectList getAllObjects(){ return game_objs; }

	// Send step event to interested objects, delete the objects
	// marked for deletion, check for collision / send collision events
	void update();

	// Determines if the given object collides with anything
	// if so, returns the object it collides with, else NULL
	GameObject* findCollision(GameObject* obj_ptr);
	// Determines if the two boxes intersect
	bool boxIntersect(Box box1, Box box2);

	// Getters, setters for private fields
	Box getBoundary(){ return world_boundary; }
	void setBoundary(Box new_boundary){ world_boundary = new_boundary; }

	// Return the WorldManager instance
	static WorldManager& getInstance(){
		static WorldManager manager;
		return manager;
	}
};

#endif // __WORLDMANAGER_H__
