// GameController.h
// Brian Seney, Alfredo Porras
// IMGD3000

#ifndef __GAMECONTROLLER_H__
#define __GAMECONTROLLER_H__

#include "Engine/GameObject.h"
#include "Engine/EventManager.h"

#include <vector>

class GameController : public GameObject{
	private:
	long int level_time;
	bool is_visible; // if the display should be drawn to the screen

	int game_mode; // 1 - 1 player, 2 - 2 player

	vector<string> level_pictures;
	vector<string> level_files;
	int current_level;

	Player* player1; // a pointer to the two players
	Player* player2;

	void winLevel(); // go onto the next level
	void loseLevel(); // restart the last level

	GameController();
	~GameController();

	// Prohibit copy, assignment
	GameController(GameController const&){}
	void operator=(GameController const&){}

	public:
	int eventHandler(Event *event_ptr);
	void draw();

	// Load a level from the levels folder into memory
	// createLevel displays a sprite first, and then calls loadLevel
	int createLevel(string sprite_file, string level_name);
	int loadLevel(string level_name);
	int loadLevelObjects(string level_name);

	void startGame(int game_type); // Loads the first level

	// Return the GameController instance
	static GameController& getInstance(){
		static GameController g_controller;
		return g_controller;
	}
};

#endif // __GAMECONTROLLER_H__
