// Opponent.cpp
// Alfredo Porras, Brian Seney
// IMGD 3000

#include "Opponent.h"

#include "Engine/WorldManager.h"
#include "Engine/EventManager.h"
#include "Engine/GraphicsManager.h"
#include "Engine/EventStep.h"
#include "Engine/EventCollision.h"

#include "Ball.h"
#include "Bullet.h"

Opponent::Opponent() {
	// initialize fields
	health = 5;

	do_attack = false;
	setType(OPPONENT_TYPE);
	move_slowdown = 5;
	move_countdown = move_slowdown;
	direction = 1;
	direction_count = 10;
	direction_countdown = direction_count;

	display_char = '@';
	color = 3;

	// add the object to the world
	WorldManager &world_manager = WorldManager::getInstance();
	world_manager.addObj(this);

	// register for events
	EventManager &event_manager = EventManager::getInstance();
	event_manager.registerInterest(this, STEP_EVENT);
	event_manager.registerInterest(this, COLLISION_EVENT);
}

Opponent::~Opponent() {
	EventManager &event_manager = EventManager::getInstance();
	event_manager.removeInterest(this, STEP_EVENT);
	event_manager.removeInterest(this, COLLISION_EVENT);
}

int Opponent::eventHandler(Event *event_ptr) {
	if (event_ptr->getType() == STEP_EVENT) {
		if(!do_attack) {
			move();
		}
		else {
			attack(); // fix
		}
		return 1;
	}
	if (event_ptr->getType() == COLLISION_EVENT) {
		EventCollision *collision_ptr = static_cast<EventCollision*>(event_ptr);
		GameObject* obj_ptr = collision_ptr->getObj2();
		// If colliding with a weapon, take damage.
		if(obj_ptr->getType().compare( BULLET_TYPE ) == 0){
			health -= static_cast<Bullet*>(obj_ptr)->getDamage();
			if(health <= 0){
				WorldManager &world_manager = WorldManager::getInstance();
				world_manager.markForDelete(this);
			}
		}
		// If colliding with the player, destroy self.
		if(obj_ptr->getType().compare( PLAYER_TYPE ) == 0){
			WorldManager &world_manager = WorldManager::getInstance();
			world_manager.markForDelete(this);
		}
		return 1;
	}
	return 0;
}

void Opponent::move() {
	move_countdown--;
	if (move_countdown > 0) {
		return;
	}
	move_countdown = move_slowdown;

	Position new_pos;
	switch(direction) {
		case 1:
		{
			new_pos.set_x( pos.get_x() );
			new_pos.set_y( pos.get_y() + 1);
			break;
		}
		case 2:
		{
			new_pos.set_x( pos.get_x() - 1);
			new_pos.set_y( pos.get_y() );
			break;
		}
		case 3:
		{
			new_pos.set_x( pos.get_x() );
			new_pos.set_y( pos.get_y() - 1);
			break;
		}
		case 4:
		{
			new_pos.set_x( pos.get_x() + 1);
			new_pos.set_y( pos.get_y() );
			break;
		}

	}

	WorldManager &world_manager = WorldManager::getInstance();
	world_manager.moveObj(this, new_pos);

	
	/* If the object could not be moved, wait until path is free to move
	if(world_manager.moveObj(this, new_pos) == 1){
		return;
	} */

	direction_countdown--;
	if (direction_countdown > 0) {
		return;
	}
	direction_countdown = direction_count;

	if(direction == 4)
		direction = 1;
	else
		direction++;
}

void Opponent::attack() {
	WorldManager &world_manager = WorldManager::getInstance();
	if (target.get_x() > pos.get_x()) {
		world_manager.moveObj(this, Position(pos.get_x() + 1, pos.get_y()));
		return;
	}
	if (target.get_x() < pos.get_x()) {
		world_manager.moveObj(this, Position(pos.get_x() - 1, pos.get_y()));
		return;
	}
	if (target.get_y() > pos.get_y()) {
		world_manager.moveObj(this, Position(pos.get_x(), pos.get_y() + 1));
		return;
	}
	if (target.get_y() < pos.get_y()) {
		world_manager.moveObj(this, Position(pos.get_x(), pos.get_y() - 1));
		return;
	}
}

void Opponent::draw(){
	GraphicsManager &graph_manager = GraphicsManager::getInstance();
	graph_manager.drawCh(pos, display_char, color);
}
