// OpponentBallSteal.cpp
// Brian Seney, Alfredo Porras
// IMGD3000

#include "OpponentBallSteal.h"

#include "Engine/WorldManager.h"
#include "Engine/EventManager.h"
#include "Engine/EventStep.h"
#include "Engine/EventCollision.h"

#include "EventTargetMovement.h"
#include "EventBallMovement.h"
#include "EventBallSteal.h"

OpponentBallSteal::OpponentBallSteal(){
	display_char = '&';
	color = 5;

	// the player wont take damage from this
	object_type = OPPONENT_BALL_STEAL;

	target = Position(-1, -1);
	player_target = Position(-1, -1); // the closest player to this object

	holding_ball = NULL;

	move_slowdown = 15;
	move_countdown = 0;

	health = 10;

	WorldManager &world_manager = WorldManager::getInstance();
	world_manager.addObj(this);

	EventManager &event_manager = EventManager::getInstance();
	event_manager.registerInterest(this, STEP_EVENT);
	event_manager.registerInterest(this, COLLISION_EVENT);
	event_manager.registerInterest(this, BALLMOVEMENT_EVENT);
	event_manager.registerInterest(this, TARGETMOVEMENT_EVENT);
	event_manager.registerInterest(this, BALL_STEAL_EVENT);
}

OpponentBallSteal::~OpponentBallSteal(){
	EventManager &event_manager = EventManager::getInstance();
	event_manager.removeInterest(this, STEP_EVENT);
	event_manager.removeInterest(this, COLLISION_EVENT);
	event_manager.removeInterest(this, BALLMOVEMENT_EVENT);
	event_manager.removeInterest(this, TARGETMOVEMENT_EVENT);
	event_manager.removeInterest(this, BALL_STEAL_EVENT);
}

int OpponentBallSteal::eventHandler(Event *event_ptr){
	if(event_ptr->getType().compare( STEP_EVENT ) == 0){
		// slow the movement down
		move_countdown--;
		if(move_countdown <= 0){
			move_countdown = move_slowdown;
		} else {
			return 1;
		}
		// if without the ball, move towards the ball
		if(!holding_ball){
			if(target.get_x() != -1){
				if( abs(pos.get_x() - target.get_x()) >
				    abs(pos.get_y() - target.get_y())){
					if(pos.get_x() > target.get_x()){
						move(-1, 0);
					} else {
						move(1, 0);
					}
				} else {
					if(pos.get_y() > target.get_y()){
						move(0, -1);
					} else {
						move(0, 1);
					}
				}
			}
		// if with the ball, move away from the closest player
		} else {
			if(player_target.get_x() != -1){
				if( abs(pos.get_x() - player_target.get_x()) >
				    abs(pos.get_y() - player_target.get_y())){
					if(pos.get_x() > player_target.get_x()){
						move(1, 0);
					} else {
						move(-1, 0);
					}
				} else {
					if(pos.get_y() > player_target.get_y()){
						move(0, 1);
					} else {
						move(0, -1);
					}
				}
			}
		}
		return 1;
	} else if(event_ptr->getType().compare( COLLISION_EVENT ) == 0){
		EventCollision *collision_ptr = static_cast<EventCollision*>(event_ptr);
		GameObject* obj_ptr = collision_ptr->getObj2();
		// If colliding with a weapon, take damage.
		if(obj_ptr->getType().compare( BULLET_TYPE ) == 0){
			health -= static_cast<Bullet*>(obj_ptr)->getDamage();
			if(health <= 0){
				WorldManager &world_manager = WorldManager::getInstance();
				world_manager.markForDelete(this);
			}
		// If colliding with the ball, take it
		} else if(obj_ptr->getType().compare( BALL_TYPE ) == 0){
			holding_ball = static_cast<Ball*>(collision_ptr->getObj2());
			// Let other players know to drop the ball
			EventManager &event_manager = EventManager::getInstance();
			event_manager.sendEvent( new EventBallSteal(holding_ball, this) );
		}
		return 1;
	} else if(event_ptr->getType().compare( BALLMOVEMENT_EVENT ) == 0){
		EventBallMovement* movement_ptr = static_cast<EventBallMovement*>(event_ptr);
		target = movement_ptr->getTargetPos();
		return 1;
	} else if(event_ptr->getType().compare( TARGETMOVEMENT_EVENT ) == 0){
		EventTargetMovement *event_target_ptr = static_cast <EventTargetMovement *> (event_ptr);
		Position temp_target = event_target_ptr->getTargetPos();
		// update the target
		player_target = temp_target;
		return 1;
	} else if(event_ptr->getType().compare( BALL_STEAL_EVENT ) == 0){
		// If the ball being stolen is this one, release the ball
		EventBallSteal *steal_ptr = static_cast<EventBallSteal*>(event_ptr);
		// If holding ball is not null and is the  ball being stolen
		if(holding_ball && holding_ball == steal_ptr->getStolenBall()){
			// If the player is not stealing the ball from itself
			if(steal_ptr->getStolenPlayer() == this){
			} else {
				move_countdown = 8 * move_slowdown; // wait to let the player retake the ball
				holding_ball = NULL;
			}
		}
		return 1;
	}
	return 0;
}

void OpponentBallSteal::move(int x, int y){
	WorldManager &world_manager = WorldManager::getInstance();
	if(!holding_ball){
		world_manager.moveObj(this, Position(pos.get_x() + x, pos.get_y() + y));
	} else {
		// if the ball can be moved in front of the position (move returns 0), move
		if(holding_ball->moveAbsolute(pos.get_x() + x, pos.get_y() + y) == 0){
			if(holding_ball->moveAbsolute(pos.get_x() + 2 * x, pos.get_y() + 2 * y) == 0){
				world_manager.moveObj(this, Position(pos.get_x() + x, pos.get_y() + y));
			}
		}
	}
}
