// Alfredo Porras
// Brian Seney
// IMGD 3000

#include "OpponentHunter.h"
#include "Engine/WorldManager.h"
#include "Engine/EventManager.h"
#include "Engine/EventStep.h"
#include "Engine/PathSearch.h"
#include "EventTargetMovement.h"

OpponentHunter::OpponentHunter() {
	hunting = false;
	sight_radius = 15;

	display_char = 'H';
	color = 2;

	EventManager &event_manager = EventManager::getInstance();
	event_manager.registerInterest(this, STEP_EVENT);
	event_manager.registerInterest(this, TARGETMOVEMENT_EVENT);
	event_manager.registerInterest(this, COLLISION_EVENT);
}

OpponentHunter::OpponentHunter(Position target) {
	hunting = false;
	sight_radius = 15;
	EventManager &event_manager = EventManager::getInstance();
	event_manager.registerInterest(this, STEP_EVENT);
	event_manager.registerInterest(this, TARGETMOVEMENT_EVENT);
	event_manager.registerInterest(this, COLLISION_EVENT);
}

OpponentHunter::~OpponentHunter() {
	EventManager &event_manager = EventManager::getInstance();
	event_manager.removeInterest(this, STEP_EVENT);
	event_manager.removeInterest(this, TARGETMOVEMENT_EVENT);
	event_manager.removeInterest(this, COLLISION_EVENT);
}

int OpponentHunter::eventHandler(Event *event_ptr) {
	if(event_ptr->getType() == STEP_EVENT) {
		move();
		return 1;
	}
	if (event_ptr->getType() == TARGETMOVEMENT_EVENT) {
		if (!hunting) {
			EventTargetMovement *p_etm = static_cast <EventTargetMovement *> (event_ptr);
			if(isClose(p_etm->getTargetPos())){
				hunting = true;
				PathSearch *p_path_finder = new PathSearch();
				p_path_finder->findPath(pos, p_etm->getTargetPos(), &path);
				delete p_path_finder;
			}
		}

		return 1;
	}
	if (event_ptr->getType() == COLLISION_EVENT) {
		EventCollision *collision_ptr = static_cast<EventCollision*>(event_ptr);
		GameObject* obj_ptr = collision_ptr->getObj2();
		// If colliding with a weapon, take damage.
		if(obj_ptr->getType().compare( BULLET_TYPE ) == 0){
			health -= static_cast<Bullet*>(obj_ptr)->getDamage();
			if(health <= 0){
				WorldManager &world_manager = WorldManager::getInstance();
				world_manager.markForDelete(this);
			}
		}
		// If colliding with the player, destroy self.
		if(obj_ptr->getType().compare( PLAYER_TYPE ) == 0){
			WorldManager &world_manager = WorldManager::getInstance();
			world_manager.markForDelete(this);
		}
		return 1;
	}
	return 0;
}

bool OpponentHunter::isClose(Position targ) {
	if (abs(targ.get_x() - pos.get_x()) < sight_radius) {
		if (abs(targ.get_y() - pos.get_y()) < sight_radius) {
			return true;
		}
	}
	return false;
}

void OpponentHunter::move() {
	move_countdown--;
	if (move_countdown > 0) {
		return;
	}
	move_countdown = move_slowdown;

	if(hunting) {
		if(!path.empty()){
			WorldManager &world_manager = WorldManager::getInstance();
			// is there an unforseen obstacle in the way?
			// if there is, forget path
			// and wait for target to be in sight again
			if(world_manager.moveObj(this, path.top()) == 0){
				path.pop();
				if(path.empty()){
					hunting = false;
				}
			}
			else { // unexpected collision
				clearPath();
				hunting = false;
			}
		}
		else {
			hunting = false;
		}
	}
}

void OpponentHunter::clearPath() {
	while(!path.empty()) {
		path.pop();
	}
}
