// OpponentSpawner.cpp
// Brian Seney, Alfredo Porras
// IMGD3000

#include "OpponentSpawner.h"
#include "OpponentZombie.h"

#include "Engine/ResourceManager.h"

OpponentSpawner::OpponentSpawner(){
	sprite_countdown = 0;
	sprite_slowdown = 5;
	sprite_index = 0;

	move_slowdown = 20;
	move_countdown = 0;

	ResourceManager &resource_manager = ResourceManager::getInstance();
	sprite_ptr = resource_manager.loadSprite("Sprites/OpponentSpawner.txt");
	obj_box = Box(pos, sprite_ptr->getWidth(), sprite_ptr->getHeight());
}

OpponentSpawner::~OpponentSpawner(){
	// Spawn zombie enemies
}

int OpponentSpawner::eventHandler(Event *event_ptr) {
	if (event_ptr->getType() == STEP_EVENT) {
		move_countdown--;
		if(move_countdown <= 0){
			move_countdown = move_slowdown;
			move();
		}
		return 1;
	}
	if (event_ptr->getType() == COLLISION_EVENT) {
		EventCollision *collision_ptr = static_cast<EventCollision*>(event_ptr);
		GameObject* obj_ptr = collision_ptr->getObj2();
		// If colliding with a weapon, take damage.
		if(obj_ptr->getType().compare( BULLET_TYPE ) == 0){
			health -= static_cast<Bullet*>(obj_ptr)->getDamage();
			if(health <= 0){
				WorldManager &world_manager = WorldManager::getInstance();
				world_manager.markForDelete(this);

				// Spawn some enemies
				if(move_slowdown > 0){
					OpponentZombie* zombie_ptr = new OpponentZombie();
					zombie_ptr->setPos(pos.get_x() + 1, pos.get_y() + 1);
					zombie_ptr = new OpponentZombie();
					zombie_ptr->setPos(pos.get_x() + 3, pos.get_y() + 1);
					zombie_ptr = new OpponentZombie();
					zombie_ptr->setPos(pos.get_x() + 3, pos.get_y() + 3);
					zombie_ptr = new OpponentZombie();
					zombie_ptr->setPos(pos.get_x() + 1, pos.get_y() + 3);
				}
				move_slowdown = -1; //  makes sure enemies are only spawned once		
			}
		}
		// If colliding with a weapon, take damage.
		return 1;
	}
}

void OpponentSpawner::move(){
	Position new_pos;
	direction = (random() % 4) + 1;
	switch(direction) {
		case 1:
		{
			new_pos.set_x( pos.get_x() );
			new_pos.set_y( pos.get_y() + 1);
			break;
		}
		case 2:
		{
			new_pos.set_x( pos.get_x() - 1);
			new_pos.set_y( pos.get_y() );
			break;
		}
		case 3:
		{
			new_pos.set_x( pos.get_x() );
			new_pos.set_y( pos.get_y() - 1);
			break;
		}
		case 4:
		{
			new_pos.set_x( pos.get_x() + 1);
			new_pos.set_y( pos.get_y() );
			break;
		}

	}

	// If the object could not be moved, try next time
	WorldManager &world_manager = WorldManager::getInstance();
	if(world_manager.moveObj(this, new_pos) == 1){
		return;
	}
}

void OpponentSpawner::draw(){
	GraphicsManager &graph_manager = GraphicsManager::getInstance();

	// Find the frame to draw
	sprite_countdown--;
	// If ready to draw a new frame, increase the index
	if(sprite_countdown < 0){
		sprite_countdown = sprite_slowdown;
		sprite_index++;
		if(sprite_index >= sprite_ptr->getNumFrames() ){
			sprite_index = 0;
		}
	}
	graph_manager.drawFrame(pos, sprite_ptr->getFrame(sprite_index));
}
