// OpponentZombie.cpp
// Alfredo Porras, Brian Seney
// IMGD 3000

#include <cmath>

#include "Engine/EventManager.h"

#include "OpponentZombie.h"
#include "EventTargetMovement.h"

OpponentZombie::OpponentZombie() {
	display_char = 'Z';
	color = 7;

	move_slowdown = 20;

	sight_radius = 15;
	EventManager &event_manager = EventManager::getInstance();
	event_manager.registerInterest(this, TARGETMOVEMENT_EVENT);
}

OpponentZombie::OpponentZombie(int sight) {
	sight_radius = sight;

	move_slowdown = 20;

	EventManager &event_manager = EventManager::getInstance();
	event_manager.registerInterest(this, TARGETMOVEMENT_EVENT);
}

OpponentZombie::~OpponentZombie() {
	EventManager &event_manager = EventManager::getInstance();
	event_manager.removeInterest(this, TARGETMOVEMENT_EVENT);
}

int OpponentZombie::eventHandler(Event *event_ptr) {
	if (event_ptr->getType() == STEP_EVENT) {
		if(!do_attack) {
			move();
		}
		else {
			attack(); // fix
		}
		return 1;
	}
	if (event_ptr->getType() == COLLISION_EVENT) {
		EventCollision *collision_ptr = static_cast<EventCollision*>(event_ptr);
		GameObject* obj_ptr = collision_ptr->getObj2();
		// If colliding with a weapon, take damage.
		if(obj_ptr->getType().compare( BULLET_TYPE ) == 0){
			health -= static_cast<Bullet*>(obj_ptr)->getDamage();
			if(health <= 0){
				WorldManager &world_manager = WorldManager::getInstance();
				world_manager.markForDelete(this);
			}
		// If colliding with the player, destroy self
		} else if(obj_ptr->getType().compare( PLAYER_TYPE ) == 0){
			WorldManager &world_manager = WorldManager::getInstance();
			world_manager.markForDelete(this);
		}
		return 1;
	}
	if (event_ptr->getType() == TARGETMOVEMENT_EVENT) {
		EventTargetMovement *p_etm = static_cast <EventTargetMovement *> (event_ptr);
		if(isClose(p_etm->getTargetPos())) {
			do_attack = true;
			target = p_etm->getTargetPos();
		}
		else {
			do_attack = false; // probably cheaper to do this every movement than checking other conditions
		}
		return 1;
	}
	
}

bool OpponentZombie::isClose(Position targ) {
	if (abs(targ.get_x() - pos.get_x()) < sight_radius) {
		if (abs(targ.get_y() - pos.get_y()) < sight_radius) {
			return true;
		}
	}
	return false;
}

void OpponentZombie::move() {
	move_countdown--;
	if (move_countdown > 0) {
		return;
	}
	move_countdown = move_slowdown;

	Position new_pos;
	direction = (random() % 4) + 1;
	switch(direction) {
		case 1:
		{
			new_pos.set_x( pos.get_x() );
			new_pos.set_y( pos.get_y() + 1);
			break;
		}
		case 2:
		{
			new_pos.set_x( pos.get_x() - 1);
			new_pos.set_y( pos.get_y() );
			break;
		}
		case 3:
		{
			new_pos.set_x( pos.get_x() );
			new_pos.set_y( pos.get_y() - 1);
			break;
		}
		case 4:
		{
			new_pos.set_x( pos.get_x() + 1);
			new_pos.set_y( pos.get_y() );
			break;
		}

	}

	// If the object could not be moved, try next time
	WorldManager &world_manager = WorldManager::getInstance();
	if(world_manager.moveObj(this, new_pos) == 1){
		return;
	}
}

void OpponentZombie::attack() {
	move_countdown--;
	if (move_countdown > 0) {
		return;
	}
	move_countdown = move_slowdown;
	
	WorldManager &world_manager = WorldManager::getInstance();
	if (target.get_x() > pos.get_x()) {
		world_manager.moveObj(this, Position(pos.get_x() + 1, pos.get_y()));
		return;
	}
	if (target.get_x() < pos.get_x()) {
		world_manager.moveObj(this, Position(pos.get_x() - 1, pos.get_y()));
		return;
	}
	if (target.get_y() > pos.get_y()) {
		world_manager.moveObj(this, Position(pos.get_x(), pos.get_y() + 1));
		return;
	}
	if (target.get_y() < pos.get_y()) {
		world_manager.moveObj(this, Position(pos.get_x(), pos.get_y() - 1));
		return;
	}
}


