// Picture.cpp
// Brian Seney
// IMGD3000

#include "Engine/GraphicsManager.h"
#include "Engine/ResourceManager.h"
#include "Engine/WorldManager.h"
#include "Engine/EventManager.h"
#include "Engine/EventStep.h"
#include "Engine/Event.h"
#include "Picture.h"

Picture::Picture(){
	startUp();
}

Picture::Picture(Sprite *spr_ptr){
	if(!spr_ptr)
		return;
	sprite_ptr = spr_ptr;

	startUp();
}

void Picture::startUp(){
	// By default the altitude will be below everything and non-solid
	altitude = -2;
	is_solid = false;

	time_to_live = 180; // Roughly 2 seconds by default

	sprite_slowdown = 4;
	sprite_countdown = sprite_slowdown;
	sprite_index = 0;

	WorldManager &world_manager = WorldManager::getInstance();
	world_manager.addObj(this);

	EventManager &event_manager = EventManager::getInstance();
	event_manager.registerInterest(this, STEP_EVENT);
}

Picture::~Picture(){
	EventManager &event_manager = EventManager::getInstance();
	event_manager.removeInterest(this, STEP_EVENT);
}

int Picture::eventHandler(Event *event_ptr){
	if(event_ptr->getType().compare( STEP_EVENT ) == 0){
		if(time_to_live > 0)
			time_to_live--;
		if(time_to_live == 0){
			WorldManager &world_manager = WorldManager::getInstance();
			world_manager.markForDelete(this);
		}
	}
}

void Picture::draw(){
	GraphicsManager &graph_manager = GraphicsManager::getInstance();

	// Find the frame to draw
	sprite_countdown--;
	// If ready to draw a new frame, increase the index
	if(sprite_countdown < 0){
		sprite_countdown = sprite_slowdown;
		sprite_index++;
		if(sprite_index >= sprite_ptr->getNumFrames() ){
			sprite_index = 0;
		}
	}
	graph_manager.drawFrame(pos, sprite_ptr->getFrame(sprite_index));
}
