// Player.h
// Brian Seney, Alfredo Porras
// IMGD3000

#ifndef __PLAYER_H__
#define __PLAYER_H__

#include "Engine/GameObject.h"
#include "Engine/Event.h"

#include "Weapon.h"
#include "Ball.h"

#include <vector>

#define PLAYER_TYPE "player"
#define HEALTH_TYPE "health"

class Player : public GameObject{
	private:
	char direction;
	Ball* holding_ball;
	int color;
	
	// 0, 1, 2, 3 - up, down, left, right
	// 4 - shoot, 5 - drop ball
	// 6 - switch weapons
	vector<int> keys;

	Position move_to;
	bool resolve_move;

	int health;
	
	bool is_enabled;

	vector<Weapon*> weapons;
	int current_weapon; // index into the weapons vector

	// Shoot the ball in the given direction
	void shootBall(float vel_x, float vel_y, float accel_x, float accel_y);

	public:
	Player();
	~Player();
	
	void setHoldingBall(Ball* new_has_ball){ holding_ball = new_has_ball; }

	char getDirection(){ return direction; }
	void setDirection(char new_direction){ direction = new_direction; }
	Weapon* getCurrentWeapon(){ return weapons[current_weapon]; }
	void setCurrentWeapon(Weapon* new_weapon);
	int getHealth(){ return health; }
	void setHealth(int new_health){ health = new_health; }
	bool isEnabled(){ return is_enabled; }
	void setEnabled( bool new_enabled ){
		is_enabled = new_enabled;
		is_solid = new_enabled;
	}
	int getColor(){ return color; }
	void setColor(int new_color){ color = new_color; }

	void setKeys( int first, ... );

	void setResolveMove(bool b){resolve_move = b;}
	void setMoveTo(Position p){move_to = p;}

	void adjustHealth(int new_health); // changes the player's health; ensures the player is alive

	// add a weapon with the given characteristics
	void addWeapon(string weapon_type, char display_char, int weapon_color, int slowdown);

	int eventHandler(Event *event_ptr);
	void draw();

	void move(int x, int y);
};

#endif // __PLAYER_H__
