// PreLevelPicture.cpp
// Brian Seney, Alfredo Porras
// IMGD3000

#include "PreLevelPicture.h"

#include "Engine/EventKeyboard.h"

PreLevelPicture::PreLevelPicture(string sprite_to_load, string level_to_load){
	// if the passed string is not the empty string
	if(sprite_to_load.length() > 0){
		ResourceManager &resource_manager = ResourceManager::getInstance();
		Sprite *spr_ptr = resource_manager.loadSprite(sprite_to_load);
		sprite_ptr = spr_ptr;

		level = level_to_load;

		time_to_live = 3;

		WorldManager &world_manager = WorldManager::getInstance();
		world_manager.addObj(this);

		EventManager &event_manager = EventManager::getInstance();
		event_manager.registerInterest(this, KEYBOARD_EVENT);
		event_manager.registerInterest(this, STEP_EVENT);
	}
}

PreLevelPicture::~PreLevelPicture(){
	EventManager &event_manager = EventManager::getInstance();
	event_manager.removeInterest(this, KEYBOARD_EVENT);
	event_manager.removeInterest(this, STEP_EVENT);

	// if the level passed to this is not the empty string
	if(level.length() > 0){
		GameController &game_controller = GameController::getInstance();
		game_controller.loadLevel(level);
	}
}

int PreLevelPicture::eventHandler(Event *event_ptr){
	if(event_ptr->getType().compare( STEP_EVENT ) == 0){
		if(time_to_live > 0)
			time_to_live--;
		return 1;
	// if a keyboard event, mark this for deletion if time to live is equal to 0
	} else if(event_ptr->getType().compare( KEYBOARD_EVENT ) == 0){
		if(time_to_live == 0){
			WorldManager &world_manager = WorldManager::getInstance();
			world_manager.markForDelete(this);
		}
		return 1;
	}
	return 0;
}
