// Spawner.cpp
// Brian Seney, Alfredo Porras
// IMGD3000

#include "Spawner.h"

#include "Engine/EventManager.h"
#include "Engine/EventStep.h"
#include "Engine/GraphicsManager.h"
#include "Engine/WorldManager.h"

#include "EventScore.h"

Spawner::Spawner(void(*fp)(int x_pos, int y_pos)){
	spawn_slowdown = -1;
	spawn_countdown = 0;

	score_spawn = false;

	function_pointer = fp;

	altitude = -1; // draw below other objects
	is_solid = false;

	WorldManager &world_manager = WorldManager::getInstance();
	world_manager.addObj(this);

	EventManager &event_manager = EventManager::getInstance();
	event_manager.registerInterest(this, STEP_EVENT);
	event_manager.registerInterest(this, SCORE_EVENT);
}

Spawner::~Spawner(){
	EventManager &event_manager = EventManager::getInstance();
	event_manager.removeInterest(this, STEP_EVENT);
	event_manager.removeInterest(this, SCORE_EVENT);
}

int Spawner::eventHandler(Event *event_ptr){
	if(event_ptr->getType().compare(STEP_EVENT) == 0){
		// If spawn countdown is defined, lower it, and
		// spawn the object is ready
		if(spawn_slowdown > -1){
			spawn_countdown--;
			if(spawn_countdown <= 0){
				function_pointer(pos.get_x(), pos.get_y());
				spawn_countdown = spawn_slowdown;
			}
		}
		return 1;
	} else if(event_ptr->getType().compare(SCORE_EVENT) == 0){
		// If responding to score events, spawn an object
		if(score_spawn){
			function_pointer(pos.get_x(), pos.get_y());
			return 1;
		}
	}
	return 0;
}

void Spawner::draw(){
	GraphicsManager &graphics_manager = GraphicsManager::getInstance();
	graphics_manager.drawCh(pos, '%', 3);
}
