// TitleScreen.cpp
// Brian Seney, Alfredo Porras
// IMGD3000

#include "TitleScreen.h"
#include "GameController.h"
#include "Engine/EventManager.h"
#include "Engine/EventKeyboard.h"
#include "Engine/EventStep.h"
#include "Engine/ResourceManager.h"
#include "Engine/WorldManager.h"
#include "Engine/GraphicsManager.h"

TitleScreen::TitleScreen(){
	setPos(0, -18);
	velocity_x = 0;
	velocity_y = 0.1;

	is_color = false;
	sprite_slowdown = 5;

	WorldManager &world_manager = WorldManager::getInstance();
	world_manager.addObj(this);

	EventManager &event_manager = EventManager::getInstance();
	event_manager.registerInterest(this, KEYBOARD_EVENT);
	event_manager.registerInterest(this, STEP_EVENT);

	ResourceManager &resource_manager = ResourceManager::getInstance();
	sprite_ptr = resource_manager.loadSprite("Sprites/TitleScreen.txt");
	// load the sprite
}

TitleScreen::~TitleScreen(){
	EventManager &event_manager = EventManager::getInstance();
	event_manager.removeInterest(this, KEYBOARD_EVENT);
	event_manager.removeInterest(this, STEP_EVENT);
}

int TitleScreen::eventHandler(Event *event_ptr){
	if(event_ptr->getType().compare(STEP_EVENT) == 0){
		if( !(pos == Position(0, 0)) ){
			velocityStep();
		} else {
			if(!is_color){ // If the color sprite has not been loaded yet
				ResourceManager &resource_manager = ResourceManager::getInstance();
				sprite_ptr = resource_manager.loadSprite("Sprites/TitleScreenColor.txt");
				is_color = true;
			}
		}
	} else 	if(event_ptr->getType().compare(KEYBOARD_EVENT) == 0){
		int key = static_cast<EventKeyboard*>(event_ptr)->getKey();
		if(key == '1'){
			GameController &game_controller = GameController::getInstance();
			game_controller.startGame(1);

			WorldManager &world_manager = WorldManager::getInstance();
			world_manager.markForDelete(this);
		} else if(key == '2'){
			GameController &game_controller = GameController::getInstance();
			game_controller.startGame(2);

			WorldManager &world_manager = WorldManager::getInstance();
			world_manager.markForDelete(this);
		} else if(key == 27){ // Escape key: end the game
			GameManager &game_manager = GameManager::getInstance();
			game_manager.stop();
		}
		return 1;
	}
	return 0;
}

void TitleScreen::draw(){
	GraphicsManager &graph_manager = GraphicsManager::getInstance();

	// Find the frame to draw
	sprite_countdown--;
	// If ready to draw a new frame, increase the index
	if(sprite_countdown < 0){
		sprite_countdown = sprite_slowdown;
		sprite_index++;
		if(sprite_index >= sprite_ptr->getNumFrames() ){
			sprite_index = 0;
		}
	}
	graph_manager.drawFrame(pos, sprite_ptr->getFrame(sprite_index));
}
