// Weapon.cpp
// Brian Seney, Alfredo Porras
// IMGD3000

#include "Weapon.h"
#include "Bomb.h"
#include "Engine/EventStep.h"
#include "Engine/EventManager.h"

Weapon::Weapon(){
	EventManager &event_manager = EventManager::getInstance();
	event_manager.registerInterest(this, STEP_EVENT);

	slowdown_count = 0;
}

Weapon::~Weapon(){
	EventManager &event_manager = EventManager::getInstance();
	event_manager.removeInterest(this, STEP_EVENT);
}

int Weapon::eventHandler(Event* event_ptr){
	if(event_ptr->getType().compare( STEP_EVENT ) == 0){
		if(slowdown_count > 0)
			slowdown_count--;
	}
}
void Weapon::shootWeapon(int pos_x, int pos_y, float vel_x, float vel_y, float accel_x, float accel_y){
	if(slowdown_count == 0){
		// ready to fire
	} else {
		return;
	}

	if( weapon_type.compare( SWORD_TYPE ) == 0){
		Sword *sword_ptr = new Sword(pos_x, pos_y);
		sword_ptr->setColor( color ); // make the sword the same color as the weapon
	} else if(weapon_type.compare( ARROW_TYPE ) == 0){
		new Arrow(pos_x, pos_y, vel_x, vel_y);
	} else if(weapon_type.compare( TRIPLE_SHOT_TYPE ) == 0){
		if(vel_x == 0){
			new Arrow(pos_x - 1, pos_y, -0.2, vel_y);
			new Arrow(pos_x, pos_y, 0, vel_y);
			new Arrow(pos_x + 1, pos_y, 0.2, vel_y);
		} else {
			new Arrow(pos_x, pos_y - 1, vel_x, -0.2);
			new Arrow(pos_x, pos_y, vel_x, 0);
			new Arrow(pos_x, pos_y + 1, vel_x, 0.2);
		}
	} else if(weapon_type.compare( BOMB_TYPE ) == 0){
		new Bomb(pos_x, pos_y, vel_x / 2, vel_y / 2);
	} else if(weapon_type.compare( TRIPLE_BOMB_TYPE ) == 0){
		if(vel_x == 0){
			new Bomb(pos_x - 1, pos_y, -0.1, vel_y / 2);
			new Bomb(pos_x, pos_y, 0, vel_y / 2);
			new Bomb(pos_x + 1, pos_y, 0.1, vel_y / 2);
		} else {
			new Bomb(pos_x, pos_y - 1, vel_x / 2, -0.1);
			new Bomb(pos_x, pos_y, vel_x / 2, 0);
			new Bomb(pos_x, pos_y + 1, vel_x / 2, 0.1);
		}
	} else {
		return;
	}

	slowdown_count = slowdown;
}
